"use strict"

const fs = require("fs-extra")
const chalk = require("chalk")
const execa = require("execa")
const inquirer = require("inquirer")
const EventEmitter = require("events")
const loadRemotePreset = require("../lib/utils/loadRemotePreset")
const writeFileTree = require("../lib/utils/writeFileTree")
const fileOperation = require("./utils/fileOperation")
const npmInstall = require("../lib/utils/npmInstall")
const { remotePresetMap } = require("./config/config")
const print = require("./utils/common/print")
const { hasGit, hasProjectGit, exit } = require("../lib/utils/common")

class Creator extends EventEmitter {
  constructor(name, context) {
    super()
    this.name = name
    this.context = context
    this.run = this.run.bind(this)
  }

  async create(options = {}, preset = null) {
    const { run, name, context } = this
    preset = await this.resolvePreset(options.preset, options.clone)
    await print.clearConsole()
    print.log(chalk.blue.bold(`gy-comon-cli v${require("../package.json").version}\n`))
    print.startSpinner(`正在创建项目 ${chalk.yellowBright(context)}.`)
    this.emit("creation", { event: "creating" })
    print.stopSpinner()

    // 输入自定义的项目名称
    const { projectName } = await inquirer.prompt([
      {
        name: "projectName",
        message: " 请输入项目的名称",
        default: "commonProject"
      }
    ])

    // 将下载的临时文件拷贝到项目中
    await fileOperation.copy(preset.tmpdir, preset.targetDir)

    // 写入项目名称到 package.json
    const packageJson = await fs.readJson(preset.targetDir + "/package.json")
    const pkg = Object.assign(packageJson, {
      name: projectName
    })

    print.log()
    print.startSpinner(`生成 ${chalk.yellowBright("package.json")} 等模板文件`)
    await writeFileTree(context, {
      "package.json": JSON.stringify(pkg, null, 2)
    })

    const shouldInitGit = this.shouldInitGit(options)
    if (shouldInitGit) {
      print.startSpinner("初始化Git仓库")
      this.emit("creation", { event: "git-init" })
      await run("git init")
      print.stopSpinner()
      print.log()
    }

    const shouldUseNpm = await this.shouldUseNpm()
    print.stopSpinner()
    print.log()
    if (!shouldUseNpm) {
      print.log(`项目创建成功 ${chalk.yellowBright(name)}，请执行如下命令：\n`)
      print.log()
      if (this.context === process.cwd()) {
        print.log()
      } else {
        print.log(chalk.cyan(` ${chalk.gray("$")} cd ${name}\n`))
      }
      print.log(chalk.cyan(` ${chalk.gray("$")} npm i\n`))
      print.log(chalk.cyan(` ${chalk.gray("$")} npm run dev\n`))
      print.log()
    } else {
      print.log("安装依赖：\n")
      await npmInstall(context, shouldUseNpm)
      print.log()
      print.log("项目创建成功，请执行如下命令：\n")
      if (this.context === process.cwd()) {
        print.log()
      } else {
        print.log(chalk.cyan(` ${chalk.gray("$")} cd ${name}\n`))
      }
      print.log(chalk.cyan(` ${chalk.gray("$")} npm run dev\n`))
    }
    print.log()
    this.emit("creation", { event: "done" })
  }

  /**
   * 远程加载文件
   * @param {*} name 文件配置名称
   * @param {*} clone 是否采用clone的方式
   * @returns
   */
  async resolvePreset(name, clone) {
    let preset
    print.log(`Remote preset ${chalk.cyan(`${remotePresetMap[name]}`)}\n`)
    print.startSpinner(`Fetching remote preset ${chalk.cyan(name)}...`)
    this.emit("creation", { event: "fetch-remote-preset" })
    try {
      preset = await loadRemotePreset(name, this.context, clone)
      print.stopSpinner()
    } catch (e) {
      print.stopSpinner()
      print.error(`Failed fetching remote preset ${chalk.cyan(name)}:`)
      throw e
    }

    if (!preset) {
      print.error(`preset "${name}" not found.`)
      exit(1)
    }
    return preset
  }

  run(command, args) {
    if (!args) {
      // eslint-disable-next-line no-extra-semi
      ;[command, ...args] = command.split(/\s+/)
    }
    return execa(command, args, { cwd: this.context })
  }

  /**
   * 是否初始化GIT仓库
   * @param {*} options
   * @returns
   */
  shouldInitGit(options) {
    if (!hasGit()) {
      return false
    }
    // --git
    if (options.forceGit) {
      return true
    }
    // --no-git
    if (options.git === false || options.git === "false") {
      return false
    }
    // default: true unless already in a git repo
    return !hasProjectGit(this.context)
  }

  /**
   * 是否使用npm下载
   */
  async shouldUseNpm() {
    const { action } = await inquirer.prompt([
      {
        name: "action",
        type: "list",
        message: "请选择下载项目依赖的方式：",
        choices: [
          { name: "手动下载依赖", value: false },
          { name: "使用npm下载依赖", value: "npm" },
          { name: "使用cnpm下载依赖", value: "cnpm" }
        ]
      }
    ])
    return action
  }
}

module.exports = Creator
