"use strict"

const fs = require("fs-extra")
const path = require("path")
const chalk = require("chalk")
const inquirer = require("inquirer")
const validatePackageName = require("validate-npm-package-name")
const print = require("./utils/common/print")
const config = require("./config/config")
const GeneratePage = require("./GeneratePage")
const { exit } = require("../lib/utils/common")

/**
 * 组件名称
 * @param {*} name 组件名称
 * @param {*} options 配置参数
 */
const add = async function (name, options) {
  const cwd = options.cwd || process.cwd()
  let targetDir = path.resolve(cwd, "src")
  if (!fs.existsSync(targetDir)) {
    console.log(chalk.yellowBright("目录下不存在src文件，将在本目录创建\n"))
    targetDir = path.resolve(cwd, "src")
  }
  console.log(`${chalk.cyan(targetDir)}\n`)
  const result = validatePackageName(name)
  if (!result.validForNewPackages) {
    // 如果所输入的不是合法的文件名称，则退出
    console.log(chalk.red(`不合法的组件名称: "${name}"`))
    result.errors &&
      result.errors.forEach(err => {
        console.log(chalk.red.dim("× " + err))
      })
    result.warnings &&
      result.warnings.forEach(warn => {
        console.log(chalk.red.dim("！" + warn))
      })
    exit(1)
  }
  let componentPath = path.resolve(targetDir, "components")
  console.log(`\n${chalk.yellowBright("文件将下载到:")} ${chalk.cyan(componentPath)}\n`)
  let availableName = await checkTemplateName(componentPath, name)
  if (!availableName) return
  const template = await inquirer.prompt([
    {
      name: "name",
      type: "list",
      message: "选择模板组件文件：",
      choices: config.templateList
    }
  ])
  let selectTemplate = template.name
  let index = config.templateList.findIndex(item => item.value === selectTemplate)
  if (index < 0) {
    console.error("模板组件不存在")
    return
  }
  // 前面完成准备工作，开始创建组件
  const generatePage = new GeneratePage(config.templateList[index], componentPath, availableName)
  await generatePage.create(options)
}

/**
 * 检查模板文件名是否存在
 * @param {*} componentPath 插入的路径
 * @param {*} name 模板名字
 * @returns
 */
const checkTemplateName = async function (componentPath, name) {
  let targetDir = path.resolve(componentPath, name)
  if (fs.existsSync(targetDir)) {
    // 检查文件存在
    const { action } = await inquirer.prompt([
      {
        name: "action",
        type: "list",
        message: `目标文件夹 ${chalk.cyan(componentPath)} 已经存在 "${name}" 文件，请选择：`,
        choices: [
          { name: "修改文件名称", value: "modify" },
          { name: "取消创建", value: false }
        ]
      }
    ])
    if (!action) return false // 取消创建则返回false
    if (action === "modify") {
      // 输入自定义的项目名称
      const customfile = await inquirer.prompt([
        {
          name: "name",
          message: "请输入项目的名称"
        }
      ])
      print.clearConsole()
      print.log()
      return checkTemplateName(componentPath, customfile.name) // 重新检查一遍
    }
  }
  return name // 返回可用的名称
}

module.exports = {
  add: (...args) => {
    add(...args).catch(err => {
      print.stopSpinner(false)
      print.error(err)
    })
  }
}
